@@ -32,14 +32,21 @@ handle_node_req(#httpd{path_parts=[_, <<"_local">>]}=Req) ->
 handle_node_req(#httpd{path_parts=[A, <<"_local">>|Rest]}=Req) ->
     handle_node_req(Req#httpd{path_parts=[A, node()] ++ Rest});
 % GET /_node/$node/_versions
-handle_node_req(#httpd{method='GET', path_parts=[_, Node, <<"_versions">>]}=Req) ->
-    send_json(Req, 200, {[
-        {erlang_version, ?l2b(?COUCHDB_ERLANG_VERSION)},
-        {javascript_engine, {[
-            {name, <<"spidermonkey">>},
-            {version, couch_server:get_spidermonkey_version()}
-        ]}}
-    ]});
+handle_node_req(#httpd{method='GET', path_parts=[_, _Node, <<"_versions">>]}=Req) ->
+    IcuVer = couch_ejson_compare:get_icu_version(),
+    UcaVer = couch_ejson_compare:get_uca_version(),
+    send_json(Req, 200, #{
+        erlang_version => ?l2b(?COUCHDB_ERLANG_VERSION),
+        collation_driver => #{
+            name => <<"libicu">>,
+            library_version => version_tuple_to_str(IcuVer),
+            collation_algorithm_version => version_tuple_to_str(UcaVer)
+        },
+        javascript_engine => #{
+            name => <<"spidermonkey">>,
+            version => couch_server:get_spidermonkey_version()
+        }
+    });
 handle_node_req(#httpd{path_parts=[_, _Node, <<"_versions">>]}=Req) ->
     send_method_not_allowed(Req, "GET");
 
@@ -322,3 +329,10 @@ run_queues() ->
             [DCQ | SQs] = lists:reverse(statistics(run_queue_lengths)),
             {lists:sum(SQs), DCQ}
     end.
+
+version_tuple_to_str(Version) when is_tuple(Version) ->
+    List1 = tuple_to_list(Version),
+    IsZero = fun(N) -> N == 0 end,
+    List2 = lists:reverse(lists:dropwhile(IsZero, lists:reverse(List1))),
+    List3 = [erlang:integer_to_list(N) || N <- List2],
+    ?l2b(lists:join(".", List3)).
