@@ -24,9 +24,20 @@
 
 %% Helper macro for declaring children of supervisor
 -define(CHILD(I, Type), {I, {I, start_link, []}, permanent, 100, Type, [I]}).
+-define(DEFAULT_BACKLOG, 512).
+-define(DEFAULT_SERVER_OPTIONS, "[{recbuf, undefined}]").
 
 start_link(Args) ->
-    supervisor:start_link({local,?MODULE}, ?MODULE, Args).
+    case supervisor:start_link({local, ?MODULE}, ?MODULE, Args) of
+        {ok, _} = Resp ->
+            notify_started(),
+            notify_uris(),
+            write_uris(),
+            Resp;
+        Else ->
+            notify_error(Else),
+            Else
+    end.
 
 init([]) ->
     Children = [
@@ -66,8 +77,9 @@ settings() ->
     [
         {bind_address, config:get("chttpd", "bind_address")},
         {port, config:get("chttpd", "port")},
-        {backlog, config:get("chttpd", "backlog")},
-        {server_options, config:get("chttpd", "server_options")}
+        {backlog, config:get_integer("chttpd", "backlog", ?DEFAULT_BACKLOG)},
+        {server_options, config:get("chttpd",
+            "server_options", ?DEFAULT_SERVER_OPTIONS)}
     ].
 
 maybe_replace(Key, Value, Settings) ->
@@ -95,3 +107,62 @@ append_if_set({Key, Value}, Opts) ->
         "The value for `~s` should be string convertable "
         "to integer which is >= 0 (got `~p`)", [Key, Value]),
     Opts.
+
+notify_started() ->
+    couch_log:info("Apache CouchDB has started. Time to relax.~n", []).
+
+notify_error(Error) ->
+    couch_log:error("Error starting Apache CouchDB:~n~n    ~p~n~n", [Error]).
+
+notify_uris() ->
+    lists:foreach(fun(Uri) ->
+        couch_log:info("Apache CouchDB has started on ~s", [Uri])
+    end, get_uris()).
+
+write_uris() ->
+    case config:get("couchdb", "uri_file", undefined) of
+        undefined ->
+            ok;
+        UriFile ->
+            Lines = [io_lib:format("~s~n", [Uri]) || Uri <- get_uris()],
+            write_file(UriFile, Lines)
+    end.
+
+get_uris() ->
+    Ip = config:get("chttpd", "bind_address"),
+    lists:flatmap(fun(Uri) ->
+        case get_uri(Uri, Ip) of
+            undefined -> [];
+            Else -> [Else]
+        end
+    end, [chttpd, couch_httpd, https]).
+
+get_uri(Name, Ip) ->
+    case get_port(Name) of
+        undefined ->
+            undefined;
+        Port ->
+            io_lib:format("~s://~s:~w/", [get_scheme(Name), Ip, Port])
+    end.
+
+get_scheme(chttpd) -> "http";
+get_scheme(couch_httpd) -> "http";
+get_scheme(https) -> "https".
+
+get_port(Name) ->
+    try
+        mochiweb_socket_server:get(Name, port)
+    catch
+        exit:{noproc, _} ->
+            undefined
+    end.
+
+write_file(FileName, Contents) ->
+    case file:write_file(FileName, Contents) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            Args = [FileName, file:format_error(Reason)],
+            couch_log:error("Failed ot write ~s :: ~s", Args),
+            throw({error, Reason})
+    end.
