@@ -69,9 +69,6 @@ all_test_() ->
                     fun should_return_404_for_delete_att_on_notadoc/1,
                     fun should_return_409_for_del_att_without_rev/1,
                     fun should_return_200_for_del_att_with_rev/1,
-                    fun should_not_send_csp_header_with_att_by_default/1,
-                    fun should_send_csp_header_with_att_when_configured/1,
-                    fun should_send_not_csp_header_with_att_when_no_config/1,
                     fun should_return_409_for_put_att_nonexistent_rev/1,
                     fun should_return_update_seq_when_set_on_all_docs/1,
                     fun should_not_return_update_seq_when_unset_on_all_docs/1,
@@ -212,73 +209,6 @@ should_return_200_for_del_att_with_rev(Url) ->
     end)}.
 
 
-should_not_send_csp_header_with_att_by_default(Url) ->
-  {timeout, ?TIMEOUT, ?_test(begin
-      {ok, RC, _, _} = test_request:put(
-          Url ++ "/testdoc5",
-          [?CONTENT_JSON, ?AUTH],
-          jiffy:encode(attachment_doc())
-      ),
-      ?assertEqual(201, RC),
-
-      {ok, _, Headers, _} = test_request:get(
-          Url ++ "/testdoc5/file.erl",
-          [?AUTH],
-          []
-      ),
-      CSPHeader = couch_util:get_value("Content-Security-Policy", Headers),
-      ?assertEqual(undefined, CSPHeader)
-    end)}.
-
-
-should_send_csp_header_with_att_when_configured(Url) ->
-  {timeout, ?TIMEOUT, ?_test(begin
-      {ok, RC, _, _} = test_request:put(
-          Url ++ "/testdoc51",
-          [?CONTENT_JSON, ?AUTH],
-          jiffy:encode(attachment_doc())
-      ),
-      ?assertEqual(201, RC),
-
-      config:set_boolean("csp", "attachments_enable", true, _Persist=false),
-
-      {ok, _, Headers, _} = test_request:get(
-          Url ++ "/testdoc51/file.erl",
-          [?AUTH],
-          []
-      ),
-      CSPHeader = couch_util:get_value("Content-Security-Policy", Headers),
-      ?assertEqual("sandbox", CSPHeader),
-
-      config:delete("csp", "attachments_enable", _Persist=false)
-
-    end)}.
-
-
-should_send_not_csp_header_with_att_when_no_config(Url) ->
-  {timeout, ?TIMEOUT, ?_test(begin
-      {ok, RC, _, _} = test_request:put(
-          Url ++ "/testdoc6",
-          [?CONTENT_JSON, ?AUTH],
-          jiffy:encode(attachment_doc())
-      ),
-      ?assertEqual(201, RC),
-
-      config:set_boolean("csp", "attachments_enable", false, _Persist=false),
-
-      {ok, _, Headers, _} = test_request:get(
-          Url ++ "/testdoc6/file.erl",
-          [?AUTH],
-          []
-      ),
-      CSPHeader = couch_util:get_value("Content-Security-Policy", Headers),
-      ?assertEqual(undefined, CSPHeader),
-
-      config:delete("csp", "attachments_enable", _Persist=false)
-
-    end)}.
-
-
 should_return_409_for_put_att_nonexistent_rev(Url) ->
     {timeout, ?TIMEOUT, ?_test(begin
         {ok, RC, _Headers, RespBody} = test_request:put(
@@ -329,7 +259,7 @@ should_return_correct_id_on_doc_copy(Url) ->
             [?CONTENT_JSON, ?AUTH, ?DESTHEADER1]),
         {ResultJson1} = ?JSON_DECODE(ResultBody1),
         Id1 = couch_util:get_value(<<"id">>, ResultJson1),
-        
+
         {_, _, _, ResultBody2} = test_request:copy(Url ++ "/testdoc/",
             [?CONTENT_JSON, ?AUTH, ?DESTHEADER2]),
         {ResultJson2} = ?JSON_DECODE(ResultBody2),
@@ -346,7 +276,7 @@ should_return_only_one_ok_on_doc_copy(Url) ->
        {_, _, _, ResultBody} = test_request:copy(Url ++ "/testdoc",
            [?CONTENT_JSON, ?AUTH, ?DESTHEADER1]),
        {ResultJson} = jiffy:decode(ResultBody),
-       NumOks = length(lists:filter(fun({Key, Value}) -> Key == <<"ok">> end, ResultJson)),
+       NumOks = length(lists:filter(fun({Key, _Value}) -> Key == <<"ok">> end, ResultJson)),
        [
            ?assertEqual(1, NumOks)
        ]
@@ -381,7 +311,6 @@ should_not_change_db_proper_after_rewriting_shardmap(_) ->
         TmpDb = ?tempdb(),
         Addr = config:get("chttpd", "bind_address", "127.0.0.1"),
         Port = mochiweb_socket_server:get(chttpd, port),
-        AdmPort = mochiweb_socket_server:get(couch_httpd, port),
 
         BaseUrl = lists:concat(["http://", Addr, ":", Port, "/", ?b2l(TmpDb)]),
         Url = BaseUrl ++ "?partitioned=true&q=1",
