@@ -12,7 +12,7 @@
 
 -module(ets_lru).
 -behaviour(gen_server).
--vsn(1).
+-vsn(2).
 
 
 -export([
@@ -44,11 +44,16 @@
 ]).
 
 
+-define(DEFAULT_TIME_UNIT, millisecond).
+
+-type time_value() :: integer().
+-type strict_monotonic_time() :: {time_value(), integer()}.
+
 -record(entry, {
-    key,
-    val,
-    atime,
-    ctime
+    key :: term(),
+    val :: term(),
+    atime :: strict_monotonic_time(),
+    ctime :: strict_monotonic_time()
 }).
 
 -record(st, {
@@ -56,9 +61,10 @@
     atimes,
     ctimes,
 
-    max_objs,
-    max_size,
-    max_lifetime
+    max_objs :: non_neg_integer() | undefined,
+    max_size :: non_neg_integer() | undefined,
+    max_lifetime :: non_neg_integer() | undefined,
+    time_unit = ?DEFAULT_TIME_UNIT :: atom()
 }).
 
 
@@ -164,7 +170,7 @@ handle_call({match, KeySpec, ValueSpec}, _From, St) ->
     {reply, Values, St, 0};
 
 handle_call({insert, Key, Val}, _From, St) ->
-    NewATime = erlang:now(),
+    NewATime = strict_monotonic_time(St#st.time_unit),
     Pattern = #entry{key=Key, atime='$1', _='_'},
     case ets:match(St#st.objects, Pattern) of
         [[ATime]] ->
@@ -233,7 +239,7 @@ accessed(Key, St) ->
     Pattern = #entry{key=Key, atime='$1', _='_'},
     case ets:match(St#st.objects, Pattern) of
         [[ATime]] ->
-            NewATime = erlang:now(),
+            NewATime = strict_monotonic_time(St#st.time_unit),
             Update = {#entry.atime, NewATime},
             true = ets:update_element(St#st.objects, Key, Update),
             true = ets:delete(St#st.atimes, ATime),
@@ -275,13 +281,12 @@ trim_size(#st{max_size=Max}=St) ->
 trim_lifetime(#st{max_lifetime=undefined}) ->
     ok;
 trim_lifetime(#st{max_lifetime=Max}=St) ->
-    Now = os:timestamp(),
+    Now = erlang:monotonic_time(St#st.time_unit),
     case ets:first(St#st.ctimes) of
         '$end_of_table' ->
             ok;
-        CTime ->
-            DiffInMilli = timer:now_diff(Now, CTime) div 1000,
-            case DiffInMilli > Max of
+        CTime = {Time, _} ->
+            case Now - Time > Max of
                 true ->
                     [{CTime, Key}] = ets:lookup(St#st.ctimes, CTime),
                     Pattern = #entry{key=Key, atime='$1', _='_'},
@@ -317,10 +322,10 @@ next_timeout(St) ->
     case ets:first(St#st.ctimes) of
         '$end_of_table' ->
             infinity;
-        CTime ->
-            Now = os:timestamp(),
-            DiffInMilli = timer:now_diff(Now, CTime) div 1000,
-            erlang:max(St#st.max_lifetime - DiffInMilli, 0)
+        {Time, _} ->
+            Now = erlang:monotonic_time(St#st.time_unit),
+            TimeDiff = Now - Time,
+            erlang:max(St#st.max_lifetime - TimeDiff, 0)
     end.
 
 
@@ -332,6 +337,8 @@ set_options(St, [{max_size, N} | Rest]) when is_integer(N), N >= 0 ->
     set_options(St#st{max_size=N}, Rest);
 set_options(St, [{max_lifetime, N} | Rest]) when is_integer(N), N >= 0 ->
     set_options(St#st{max_lifetime=N}, Rest);
+set_options(St, [{time_unit, T} | Rest]) when is_atom(T) ->
+    set_options(St#st{time_unit=T}, Rest);
 set_options(_, [Opt | _]) ->
     throw({invalid_option, Opt}).
 
@@ -350,3 +357,8 @@ ct_table(Name) ->
 
 table_name(Name, Ext) ->
     list_to_atom(atom_to_list(Name) ++ Ext).
+
+
+-spec strict_monotonic_time(atom()) -> strict_monotonic_time().
+strict_monotonic_time(TimeUnit) ->
+    {erlang:monotonic_time(TimeUnit), erlang:unique_integer([monotonic])}.
