@@ -15,6 +15,7 @@
 -behaviour(config_listener).
 
 -export([start_link/0, call/3]).
+-export([get_queue_lengths/0]).
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, code_change/3, terminate/2]).
 
 % config_listener api
@@ -50,8 +51,19 @@ call(Fd, Msg, Metadata) ->
             queued_call(Fd, Msg, Priority)
     end.
 
+get_queue_lengths() ->
+    gen_server:call(?MODULE, get_queue_lengths).
+
 bypass(Priority) ->
-    config:get("ioq.bypass", atom_to_list(Priority)) =:= "true".
+    case Priority of
+        os_process -> config:get_boolean("ioq.bypass", "os_process", true);
+        read -> config:get_boolean("ioq.bypass", "read", true);
+        write -> config:get_boolean("ioq.bypass", "write", true);
+        view_update -> config:get_boolean("ioq.bypass", "view_update", true);
+        shard_sync -> config:get_boolean("ioq.bypass", "shard_sync", false);
+        compaction -> config:get_boolean("ioq.bypass", "compaction", false);
+        _ -> config:get("ioq.bypass", atom_to_list(Priority)) =:= "true"
+    end.
 
 io_class({prompt, _}, _) ->
     os_process;
@@ -87,10 +99,16 @@ init(_) ->
     {ok, read_config(State)}.
 
 read_config(State) ->
-    Ratio = list_to_float(config:get("ioq", "ratio", "0.01")),
-    Concurrency = list_to_integer(config:get("ioq", "concurrency", "10")),
+    Ratio = config:get_float("ioq", "ratio", 0.01),
+    Concurrency = config:get_integer("ioq", "concurrency", 10),
     State#state{concurrency=Concurrency, ratio=Ratio}.
 
+handle_call(get_queue_lengths, _From, State) ->
+    Response = #{
+        interactive => queue:len(State#state.interactive),
+        background => queue:len(State#state.background)
+    },
+    {reply, Response, State, 0};
 handle_call(#request{}=Request, From, State) ->
     {noreply, enqueue_request(Request#request{from=From}, State), 0}.
 
